#include <bits/stdc++.h>
#define D(x) // cerr << ">> " << #x << " = " << x << endl
using namespace std;

const int INF = 1e9+7;

int ang(int deg) {
    return ((deg % 360) + 360) % 360;
}

int angDiff(int d1, int d2) {
    d1 %= 360;
    d2 %= 360;
    int diff = d1 - d2;
    if (diff > 180)
        diff -= 360;
    if (diff < -180)
    	diff += 360;
    return abs(diff);
}

struct State {
    int rotMaze;
    int angleBall;
    int layer;
    bool operator<(const State & s) const {
        return tie(rotMaze, angleBall, layer) < tie(s.rotMaze, s.angleBall, s.layer);
    }
    friend ostream & operator<<(ostream & os, const State & s) {
        return os << "Maze rotation: " << s.rotMaze << ", ball angle: " << s.angleBall << ", layer: " << s.layer;
    }
};

struct Layer {
    vector<pair<int,int>> segments;
    vector<pair<int,int>> falls; // {angle, layer}
};

using Layers = vector<Layer>;

enum Horizontal { Left, Middle, Right };
enum Vertical { Up, MiddleVer, Down };

Horizontal getHorizontal(int angle) {
    if (angle == 0 || angle == 180)
        return Middle;
    else if (angle > 0 && angle < 180)
        return Right;
    else
        return Left;
}

Vertical getVertical(int angle) {
    if (angle == 90 || angle == 270) return MiddleVer;
    if (angle > 90 && angle < 270) return Down;
    return Up;
}

bool sittingAtEdgeOfSegment(const State & s, const Layer & layer, Horizontal hor) {
    const int angle = s.angleBall;
    const auto currentSegment = find_if(
        layer.segments.begin(),
        layer.segments.end(),
        [angle] (auto segment) {
            if (segment.first < segment.second)
                return angle >= segment.first && angle <= segment.second;
            else
                return angle >= segment.first || angle <= segment.second;
        }
    );
    // pokud je jenom drop, ale uz ne segment
    if (currentSegment == layer.segments.end())
        return true;
    assert(currentSegment != layer.segments.end());

    if (hor == Middle)
        return angle == currentSegment->first || angle == currentSegment->second;
    auto res = hor == Right ? angle == currentSegment->second : angle == currentSegment->first;
    return res;
}

int isFalling(const State & s, const Layer & layer, Vertical ver, Horizontal hor) {
    // angle in terms to maze ... just to check whether the fall is met
    const int angle = s.angleBall;
    auto it = find_if(
        layer.falls.begin(),
        layer.falls.end(),
        [&s, angle, ver] (auto fall) {
            return angle == fall.first &&
                (
                        (ver == Up   && fall.second == s.layer - 1) ||
                        (ver == Down && fall.second == s.layer + 1)
                );
            }
    );
    if (it == layer.falls.end())
        return 0;

    D(sittingAtEdgeOfSegment(s, layer, hor));
    D(ver);
    if (sittingAtEdgeOfSegment(s, layer, hor) &&
        (
            (ver == Up   && it->second == s.layer - 1) ||
            (ver == Down && it->second == s.layer + 1)
        )
    )
        return it->second - s.layer;

    if (angDiff(s.angleBall + s.rotMaze, ver == Up ? 0 : 180) > 45)
        return 0;
    // realny uhel kulicky = s.angleBall + s.rotMaze
    // if (angDiff(s.angleBall + s.rotMaze, it->first + (it->second == s.layer + 1 ? 180 : 0)) > 45)
    //     return 0;
    return it->second - s.layer;
}

bool isInMove(State & s, Layers & layers) {
    const int angle = ang(s.angleBall + s.rotMaze);
    
    auto hor = getHorizontal(angle);
    auto ver = getVertical(angle);

    // je na miste odkud se pada
    // D(isFalling(s, layers[s.layer], ver));
    if (isFalling(s, layers[s.layer], ver, hor))
        return true;

    // stoji na "hrane"
    // D((hor == Middle));
    if (hor == Middle)
        return false;

    // je na konci chodby
    // D(sittingAtEdgeOfSegment(s, layers[s.layer], hor));
    if (sittingAtEdgeOfSegment(s, layers[s.layer], hor))
        return false;
    
    // sklouzavani po kraji
    return true;
}


State moveBall(State s, Layers & layers) {
    while (isInMove(s, layers)) {
        const int angle = ang(s.angleBall + s.rotMaze);
        
        auto hor = getHorizontal(angle);
        auto ver = getVertical(angle);

        // je na miste odkud se pada
        auto falling = isFalling(s, layers[s.layer], ver, hor);
        if (falling) {
            s.layer += falling;
            continue;
        }

        // stoji na "hrane"
        // if (hor == Middle)
        //     return s;

        // // sklouzava po kraji
        // if (sittingAtEdgeOfSegment(s, layers[s.layer], hor))
        //     return s;

        s.angleBall = ang(s.angleBall + (hor == Right ? +1 : -1));
    }
    return s;
}

int main() {
    int res = INF;
    Layers layers;
    int layerCnt;
    cin >> layerCnt;
    for (int layer = 0; layer < layerCnt; ++layer) {
        layers.push_back({});
        int segmentCnt, fallCnt;
        for (cin >> segmentCnt; segmentCnt--; ) {
            int start, end;
            cin >> start >> end;
            layers.back().segments.push_back({start, end});
        }
        for (cin >> fallCnt; fallCnt--; ) {
            int angle;
            cin >> angle;
            layers.back().falls.push_back({angle, layer + 1});
        }
    }
    for (int layer = layerCnt - 1; layer > 0; --layer) {
        for (auto [angle, _] : layers[layer-1].falls)
            layers[layer].falls.push_back({angle, layer - 1});
    }
    for (auto & layer : layers)
        sort(layer.falls.begin(), layer.falls.end());

    State origin = { 0 };
    State finish{ 0 };
    cin >> origin.layer >> origin.angleBall;
    cin >> finish.layer >> finish.angleBall;

    origin = moveBall(origin, layers);

    map<State, State> prev;
    map<State, int> dist{ { origin, 0 } };
    queue<State> q;
    q.push(origin);
    auto neigh = { -1, +1 };
    // set<pair<int,int>> reachable;
    while (!q.empty()) {
        auto u = q.front(); q.pop();
        // reachable.insert({u.layer, u.angleBall});
        D(u);
        if (u.angleBall == finish.angleBall && u.layer == finish.layer)
            res = min(res, dist[u]);
        for (auto n : neigh) {
            auto v = u;
            int cnt = 0;
            while (!isInMove(v, layers)) {
                v.rotMaze = ang(v.rotMaze + n);
                cnt += abs(n);
            }
            v = moveBall(v, layers);
            int distV = dist[u] + cnt;
            if (!dist.count(v) || dist[v] > distV) {
                prev[v] = u;
                dist[v] = { distV };
                q.push(v);
            }
        }
    }
    if (res == INF)
        cout << "Impossible" << endl;
    else
        cout << res << endl;
    // for (auto x : reachable)
    //     cout << x.first << " " << x.second << endl;
    return 0;
}